--- posix/tunnel.c.orig	2008-05-05 21:47:25.000000000 +0100
+++ posix/tunnel.c	2008-08-12 10:58:41.000000000 +0100
@@ -17,29 +17,65 @@
  *
  */
 
-
-
-#include <errno.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <string.h>
-#include <arpa/inet.h>
+#if defined(__FreeBSD__)
+#include <sys/cdefs.h>
+#endif
+#include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
-#include <netinet/ip.h>
-#include <netinet/udp.h>
-#include <netinet/tcp.h>
 #if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__Darwin__)
 #include <sys/sockio.h>
+#include <netinet/in.h>
+#include <netinet/in_systm.h>
 #endif
+#include <netinet/ip.h>
+#include <netinet/udp.h>
+#include <netinet/tcp.h>
 #include <net/if.h>
+#include <errno.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <string.h>
 #include <fcntl.h>        /* open(), O_RDWR */
-
+#include <arpa/inet.h>
 
 #include "../os.h"
 #include "../batman.h"
 
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__Darwin__)
+/*
+ * For compatibility with linux struct iphdr.
+ */
+struct __linux_iphdr {
+#if _BYTE_ORDER == _LITTLE_ENDIAN
+	uint8_t		ihl:4;
+	uint8_t		version:4;
+#else
+	uint8_t		version:4;
+	uint8_t		ihl:4;
+#endif
+	uint8_t		tos;
+	uint16_t	tot_len;
+	uint16_t	id;
+	uint16_t	frag_off;
+	uint8_t		ttl;
+	uint8_t		protocol;
+	uint16_t	check;
+	in_addr_t	saddr;
+	in_addr_t	daddr;
+};
+
+struct __linux_udphdr {
+	uint16_t	source;
+	uint16_t	dest;
+	uint16_t	len;
+	uint16_t	check;
+};
+
+#define iphdr __linux_iphdr
+#define udphdr __linux_udphdr
 
+#endif /* BSD */
 
 #define TUNNEL_DATA 0x01
 #define TUNNEL_IP_REQUEST 0x02
